@@ -1,5 +1,8 @@
+from typing import Dict
+
 from dagster import (
     AssetMaterialization,
+    Bool,
     EventMetadataEntry,
     InputDefinition,
     Noneable,
@@ -13,23 +16,30 @@ from dagster import (
 from dagster.config.field import Field
 from dagster.utils.backcompat import experimental
 
-from .types import DbtCliResult, DbtCliStatsResult
-from .utils import execute_dbt, get_run_results
+from .types import DbtCliOutput
+from .utils import execute_cli, parse_run_results
 
 DEFAULT_DBT_EXECUTABLE = "dbt"
 
-# the following config items correspond to flags that apply to all CLI commands
-# https://github.com/fishtown-analytics/dbt/blob/dev/marian-anderson/core/dbt/main.py#L260-L329
+# The following config fields correspond to flags that apply to all dbt CLI commands. For details
+# on dbt CLI flags, see
+# https://github.com/fishtown-analytics/dbt/blob/1f8e29276e910c697588c43f08bc881379fff178/core/dbt/main.py#L260-L329
 CLI_COMMON_FLAGS_CONFIG_SCHEMA = {
     "project-dir": Field(
         config=StringSource,
         is_required=False,
-        description="Which directory to look in for the dbt_project.yml file. Default is the current working directory and its parents.",
+        description=(
+            "Which directory to look in for the dbt_project.yml file. Default is the current "
+            "working directory and its parents."
+        ),
     ),
     "profiles-dir": Field(
         config=StringSource,
         is_required=False,
-        description="Which directory to look in for the profiles.yml file. Default = $DBT_PROFILES_DIR or $HOME/.dbt",
+        description=(
+            "Which directory to look in for the profiles.yml file. Default = $DBT_PROFILES_DIR or "
+            "$HOME/.dbt"
+        ),
     ),
     "profile": Field(
         config=StringSource,
@@ -44,7 +54,11 @@ CLI_COMMON_FLAGS_CONFIG_SCHEMA = {
     "vars": Field(
         config=Permissive({}),
         is_required=False,
-        description="Supply variables to the project. This argument overrides variables defined in your dbt_project.yml file. This argument should be a dictionary, eg. {'my_variable': 'my_value'}",
+        description=(
+            "Supply variables to the project. This argument overrides variables defined in your "
+            "dbt_project.yml file. This argument should be a dictionary, eg. "
+            "{'my_variable': 'my_value'}"
+        ),
     ),
     "bypass-cache": Field(
         config=bool,
@@ -54,14 +68,17 @@ CLI_COMMON_FLAGS_CONFIG_SCHEMA = {
     ),
 }
 
-# the following are all the config items common to all CLI solids/commands, comprised of the
-# base flags applicable to each command and other values that shouldn't be auto-converted to flags
-CLI_CONFIG_SCHEMA = {
-    **CLI_COMMON_FLAGS_CONFIG_SCHEMA,
+# The following config fields correspond to options that apply to all CLI solids, but should not be
+# formatted as CLI flags.
+CLI_COMMON_OPTIONS_CONFIG_SCHEMA = {
     "warn-error": Field(
         config=bool,
         is_required=False,
-        description="If dbt would normally warn, instead raise an exception. Examples include --models that selects nothing, deprecations, configurations with no associated models, invalid test configurations, and missing sources/refs in tests.",
+        description=(
+            "If dbt would normally warn, instead raise an exception. Examples include --models "
+            "that selects nothing, deprecations, configurations with no associated models, "
+            "invalid test configurations, and missing sources/refs in tests."
+        ),
         default_value=False,
     ),
     "dbt_executable": Field(
@@ -73,11 +90,16 @@ CLI_CONFIG_SCHEMA = {
     "ignore_handled_error": Field(
         config=bool,
         is_required=False,
-        description="When True, will not raise an exception when the dbt CLI returns error code 1. Default is False.",
+        description=(
+            "When True, will not raise an exception when the dbt CLI returns error code 1. "
+            "Default is False."
+        ),
         default_value=False,
     ),
 }
 
+CLI_CONFIG_SCHEMA = {**CLI_COMMON_FLAGS_CONFIG_SCHEMA, **CLI_COMMON_OPTIONS_CONFIG_SCHEMA}
+
 CLI_COMMON_FLAGS = set(CLI_COMMON_FLAGS_CONFIG_SCHEMA.keys())
 
 
@@ -92,14 +114,17 @@ def passthrough_flags_only(solid_config, additional_flags):
 @solid(
     description="A solid to invoke dbt run via CLI.",
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliStatsResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliOutput)],
     config_schema={
         **CLI_CONFIG_SCHEMA,
         "threads": Field(
             config=Noneable(int),
             default_value=None,
             is_required=False,
-            description="Specify number of threads to use while executing models. Overrides settings in profiles.yml.",
+            description=(
+                "Specify number of threads to use while executing models. Overrides settings "
+                "in profiles.yml."
+            ),
         ),
         "models": Field(
             config=Noneable([str]),
@@ -115,7 +140,10 @@ def passthrough_flags_only(solid_config, additional_flags):
         ),
         "full-refresh": Field(
             config=bool,
-            description="If specified, DBT will drop incremental models and fully-recalculate the incremental table from the model definition. (--full-refresh)",
+            description=(
+                "If specified, DBT will drop incremental models and fully-recalculate the "
+                "incremental table from the model definition. (--full-refresh)"
+            ),
             is_required=False,
             default_value=False,
         ),
@@ -125,12 +153,24 @@ def passthrough_flags_only(solid_config, additional_flags):
             is_required=False,
             default_value=False,
         ),
+        "yield_materializations": Field(
+            config=Bool,
+            is_required=False,
+            default_value=True,
+            description=(
+                "If True, materializations corresponding to the results of the dbt operation will "
+                "be yielded when the solid executes. Default: True"
+            ),
+        ),
     },
     tags={"kind": "dbt"},
 )
 @experimental
-def dbt_cli_run(context) -> DbtCliStatsResult:
-    logs, raw_output, return_code = execute_dbt(
+def dbt_cli_run(context) -> DbtCliOutput:
+    """This solid executes ``dbt run`` via the dbt CLI."""
+    from ..utils import generate_materializations
+
+    cli_output = execute_cli(
         context.solid_config["dbt_executable"],
         command=("run",),
         flags_dict=passthrough_flags_only(
@@ -140,40 +180,27 @@ def dbt_cli_run(context) -> DbtCliStatsResult:
         warn_error=context.solid_config["warn-error"],
         ignore_handled_error=context.solid_config["ignore_handled_error"],
     )
+    run_results = parse_run_results(context.solid_config["project-dir"])
+    cli_output_dict = {**run_results, **cli_output}
+    cli_output = DbtCliOutput.from_dict(cli_output_dict)
 
-    run_results = get_run_results(logs)
+    if context.solid_config["yield_materializations"]:
+        for materialization in generate_materializations(cli_output):
+            yield materialization
 
     yield AssetMaterialization(
-        asset_key="dbt_cli_run-shell_output",  # TODO: Perhaps derive asset key from CLI flags?
-        description="The output of a shell execution of `dbt run`.",
-        metadata_entries=[
-            EventMetadataEntry.float(
-                label="return_code",
-                value=float(return_code),
-                description="The return code of a shell exeuction of `dbt run`.",
-            ),
-            EventMetadataEntry.json(
-                label="run_results",
-                data=run_results,
-                description="The summarized results of a shell execution of `dbt run`.",
-            ),
-            EventMetadataEntry.text(
-                label="raw_output",
-                text=raw_output,
-                description="The raw output of a shell execution of `dbt run`.",
-            ),
-        ],
+        asset_key="dbt_run_cli_output",
+        description="Output from the CLI execution of `dbt run`.",
+        metadata_entries=[EventMetadataEntry.json(cli_output_dict, label="CLI Output")],
     )
 
-    yield Output(
-        DbtCliStatsResult(logs=logs, raw_output=raw_output, return_code=return_code, **run_results)
-    )
+    yield Output(cli_output)
 
 
 @solid(
     description="A solid to invoke dbt test via CLI.",
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliStatsResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliOutput)],
     config_schema={
         **CLI_CONFIG_SCHEMA,
         "data": Field(
@@ -198,7 +225,10 @@ def dbt_cli_run(context) -> DbtCliStatsResult:
             config=Noneable(int),
             default_value=None,
             is_required=False,
-            description="Specify number of threads to use while executing models. Overrides settings in profiles.yml.",
+            description=(
+                "Specify number of threads to use while executing models. Overrides settings "
+                "in profiles.yml."
+            ),
         ),
         "models": Field(
             config=Noneable([str]),
@@ -212,11 +242,16 @@ def dbt_cli_run(context) -> DbtCliStatsResult:
             is_required=False,
             description="The dbt models to exclude.",
         ),
+        "yield_materializations": Field(
+            config=Bool, is_required=False, default_value=True, description="FIXME"
+        ),
     },
+    tags={"kind": "dbt"},
 )
 @experimental
-def dbt_cli_test(context) -> DbtCliStatsResult:
-    logs, raw_output, return_code = execute_dbt(
+def dbt_cli_test(context) -> DbtCliOutput:
+    """This solid executes ``dbt test`` via the dbt CLI."""
+    cli_output = execute_cli(
         context.solid_config["dbt_executable"],
         command=("test",),
         flags_dict=passthrough_flags_only(
@@ -226,47 +261,33 @@ def dbt_cli_test(context) -> DbtCliStatsResult:
         warn_error=context.solid_config["warn-error"],
         ignore_handled_error=context.solid_config["ignore_handled_error"],
     )
+    run_results = parse_run_results(context.solid_config["project-dir"])
+    cli_output = {**run_results, **cli_output}
 
-    run_results = get_run_results(logs)
+    if context.solid_config["yield_materializations"]:
+        yield AssetMaterialization(
+            asset_key="dbt_test_cli_output",
+            description="Output from the CLI execution of `dbt test`.",
+            metadata_entries=[EventMetadataEntry.json(cli_output, label="CLI Output")],
+        )
 
-    yield AssetMaterialization(
-        asset_key="dbt_cli_test-shell_output",  # TODO: Perhaps derive asset key from CLI flags?
-        description="The output of a shell execution of `dbt test`.",
-        metadata_entries=[
-            EventMetadataEntry.float(
-                label="return_code",
-                value=float(return_code),
-                description="The return code of a shell exeuction of `dbt test`.",
-            ),
-            EventMetadataEntry.json(
-                label="run_results",
-                data=run_results,
-                description="The summarized results of a shell execution of `dbt test`.",
-            ),
-            EventMetadataEntry.text(
-                label="raw_output",
-                text=raw_output,
-                description="The raw output of a shell execution of `dbt test`.",
-            ),
-        ],
-    )
-
-    yield Output(
-        DbtCliStatsResult(logs=logs, raw_output=raw_output, return_code=return_code, **run_results)
-    )
+    yield Output(DbtCliOutput.from_dict(cli_output))
 
 
 @solid(
     description="A solid to invoke dbt snapshot via CLI.",
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=Dict)],
     config_schema={
         **CLI_CONFIG_SCHEMA,
         "threads": Field(
             config=Noneable(int),
             default_value=None,
             is_required=False,
-            description="Specify number of threads to use while executing models. Overrides settings in profiles.yml.",
+            description=(
+                "Specify number of threads to use while executing models. Overrides settings in "
+                "profiles.yml."
+            ),
         ),
         "models": Field(
             config=Noneable([str]),
@@ -280,11 +301,16 @@ def dbt_cli_test(context) -> DbtCliStatsResult:
             is_required=False,
             description="The dbt models to exclude.",
         ),
+        "yield_materializations": Field(
+            config=Bool, is_required=False, default_value=True, description="FIXME"
+        ),
     },
+    tags={"kind": "dbt"},
 )
 @experimental
-def dbt_cli_snapshot(context) -> DbtCliResult:
-    logs, raw_output, return_code = execute_dbt(
+def dbt_cli_snapshot(context) -> Dict:
+    """This solid executes ``dbt snapshot`` via the dbt CLI."""
+    cli_output = execute_cli(
         context.solid_config["dbt_executable"],
         command=("snapshot",),
         flags_dict=passthrough_flags_only(context.solid_config, ("threads", "models", "exclude")),
@@ -293,46 +319,48 @@ def dbt_cli_snapshot(context) -> DbtCliResult:
         ignore_handled_error=context.solid_config["ignore_handled_error"],
     )
 
-    yield AssetMaterialization(
-        asset_key="dbt_cli_snapshot-shell_output",  # TODO: Perhaps derive asset key from CLI flags?
-        description="The output of a shell execution of `dbt snapshot`.",
-        metadata_entries=[
-            EventMetadataEntry.float(
-                label="return_code",
-                value=float(return_code),
-                description="The return code of a shell exeuction of `dbt snapshot`.",
-            ),
-            EventMetadataEntry.text(
-                label="raw_output",
-                text=raw_output,
-                description="The raw output of a shell execution of `dbt snapshot`.",
-            ),
-        ],
-    )
+    if context.solid_config["yield_materializations"]:
+        yield AssetMaterialization(
+            asset_key="dbt_snapshot_cli_output",
+            description="Output from the CLI execution of `dbt snapshot`.",
+            metadata_entries=[EventMetadataEntry.json(cli_output, label="CLI Output")],
+        )
 
-    yield Output(DbtCliResult(logs=logs, raw_output=raw_output, return_code=return_code))
+    yield Output(cli_output)
 
 
 @solid(
     description="A solid to invoke dbt run-operation via CLI.",
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=Dict)],
     config_schema={
         **CLI_CONFIG_SCHEMA,
         "macro": Field(
             config=StringSource,
-            description="Specify the macro to invoke. dbt will call this macro with the supplied arguments and then exit.",
+            description=(
+                "Specify the macro to invoke. dbt will call this macro with the supplied "
+                "arguments and then exit."
+            ),
         ),
         "args": Field(
             config=Permissive({}),
             is_required=False,
-            description="Supply arguments to the macro. This dictionary will be mapped to the keyword arguments defined in the selected macro. This argument should be a dictionary, eg. {'my_variable': 'my_value'}",
+            description=(
+                "Supply arguments to the macro. This dictionary will be mapped to the keyword "
+                "arguments defined in the selected macro. This argument should be a dictionary, "
+                "eg. {'my_variable': 'my_value'}"
+            ),
+        ),
+        "yield_materializations": Field(
+            config=Bool, is_required=False, default_value=True, description="FIXME"
         ),
     },
+    tags={"kind": "dbt"},
 )
 @experimental
-def dbt_cli_run_operation(context) -> DbtCliResult:
-    logs, raw_output, return_code = execute_dbt(
+def dbt_cli_run_operation(context) -> Dict:
+    """This solid executes ``dbt run-operation`` via the dbt CLI."""
+    cli_output = execute_cli(
         context.solid_config["dbt_executable"],
         command=("run-operation", context.solid_config["macro"]),
         flags_dict=passthrough_flags_only(context.solid_config, ("args",)),
@@ -341,30 +369,20 @@ def dbt_cli_run_operation(context) -> DbtCliResult:
         ignore_handled_error=context.solid_config["ignore_handled_error"],
     )
 
-    yield AssetMaterialization(
-        asset_key="dbt_cli_run_operation-shell_output",  # TODO: Perhaps derive asset key from CLI flags?
-        description="The output of a shell execution of `dbt run-operation`.",
-        metadata_entries=[
-            EventMetadataEntry.float(
-                label="return_code",
-                value=float(return_code),
-                description="The return code of a shell exeuction of `dbt run-operation`.",
-            ),
-            EventMetadataEntry.text(
-                label="raw_output",
-                text=raw_output,
-                description="The raw output of a shell execution of `dbt run-operation`.",
-            ),
-        ],
-    )
+    if context.solid_config["yield_materializations"]:
+        yield AssetMaterialization(
+            asset_key="dbt_run_operation_cli_output",
+            description="Output from the CLI execution of `dbt run-operation`.",
+            metadata_entries=[EventMetadataEntry.json(cli_output, label="CLI Output")],
+        )
 
-    yield Output(DbtCliResult(logs=logs, raw_output=raw_output, return_code=return_code))
+    yield Output(cli_output)
 
 
 @solid(
     description="A solid to invoke dbt source snapshot-freshness via CLI.",
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=Dict)],
     config_schema={
         **CLI_CONFIG_SCHEMA,
         "select": Field(
@@ -376,19 +394,30 @@ def dbt_cli_run_operation(context) -> DbtCliResult:
         "output": Field(
             config=StringSource,
             is_required=False,
-            description="Specify the output path for the json report. By default, outputs to target/sources.json",
+            description=(
+                "Specify the output path for the json report. By default, outputs to "
+                "target/sources.json"
+            ),
         ),
         "threads": Field(
             config=Noneable(int),
             default_value=None,
             is_required=False,
-            description="Specify number of threads to use while executing models. Overrides settings in profiles.yml.",
+            description=(
+                "Specify number of threads to use while executing models. Overrides "
+                "settings in profiles.yml."
+            ),
+        ),
+        "yield_materializations": Field(
+            config=Bool, is_required=False, default_value=True, description="FIXME"
         ),
     },
+    tags={"kind": "dbt"},
 )
 @experimental
-def dbt_cli_snapshot_freshness(context) -> DbtCliResult:
-    logs, raw_output, return_code = execute_dbt(
+def dbt_cli_snapshot_freshness(context) -> Dict:
+    """This solid executes ``dbt source snapshot-freshness`` via the dbt CLI."""
+    cli_output = execute_cli(
         context.solid_config["dbt_executable"],
         command=("source", "snapshot-freshness"),
         flags_dict=passthrough_flags_only(context.solid_config, ("select", "output", "threads")),
@@ -397,30 +426,20 @@ def dbt_cli_snapshot_freshness(context) -> DbtCliResult:
         ignore_handled_error=context.solid_config["ignore_handled_error"],
     )
 
-    yield AssetMaterialization(
-        asset_key="dbt_cli_snapshot_freshness-shell_output",  # TODO: Perhaps derive asset key from CLI flags?
-        description="The output of a shell execution of `dbt source snapshot-freshness`.",
-        metadata_entries=[
-            EventMetadataEntry.float(
-                label="return_code",
-                value=float(return_code),
-                description="The return code of a shell exeuction of `dbt source snapshot-freshness`.",
-            ),
-            EventMetadataEntry.text(
-                label="raw_output",
-                text=raw_output,
-                description="The raw output of a shell execution of `dbt source snapshot-freshness`.",
-            ),
-        ],
-    )
+    if context.solid_config["yield_materializations"]:
+        yield AssetMaterialization(
+            asset_key="dbt_source_snapshot-freshness_cli_output",
+            description="Output from the CLI execution of `dbt source snapshot-freshness`.",
+            metadata_entries=[EventMetadataEntry.json(cli_output, label="CLI Output")],
+        )
 
-    yield Output(DbtCliResult(logs=logs, raw_output=raw_output, return_code=return_code))
+    yield Output(cli_output)
 
 
 @solid(
     description="A solid to invoke dbt compile via CLI.",
     input_defs=[InputDefinition(name="start_after", dagster_type=Nothing)],
-    output_defs=[OutputDefinition(name="result", dagster_type=DbtCliResult)],
+    output_defs=[OutputDefinition(name="result", dagster_type=Dict)],
     config_schema={
         **CLI_CONFIG_SCHEMA,
         "parse-only": Field(config=bool, is_required=False, default_value=False,),
@@ -428,11 +447,17 @@ def dbt_cli_snapshot_freshness(context) -> DbtCliResult:
             config=Noneable(int),
             default_value=None,
             is_required=False,
-            description="Specify number of threads to use while executing models. Overrides settings in profiles.yml.",
+            description=(
+                "Specify number of threads to use while executing models. Overrides settings "
+                "in profiles.yml."
+            ),
         ),
         "no-version-check": Field(
             config=bool,
-            description="Skip the check that dbt's version matches the one specified in the dbt_project.yml file ('require-dbt-version')",
+            description=(
+                "Skip the check that dbt's version matches the one specified in the "
+                "dbt_project.yml file ('require-dbt-version')"
+            ),
             is_required=False,
             default_value=False,
         ),
@@ -458,19 +483,30 @@ def dbt_cli_snapshot_freshness(context) -> DbtCliResult:
             config=Noneable([str]),
             default_value=None,
             is_required=False,
-            description="If set, use the given directory as the source for json files to compare with this project.",
+            description=(
+                "If set, use the given directory as the source for json files to compare with "
+                "this project."
+            ),
         ),
         "full-refresh": Field(
             config=bool,
-            description="If specified, DBT will drop incremental models and fully-recalculate the incremental table from the model definition. (--full-refresh)",
+            description=(
+                "If specified, DBT will drop incremental models and fully-recalculate "
+                "the incremental table from the model definition. (--full-refresh)"
+            ),
             is_required=False,
             default_value=False,
         ),
+        "yield_materializations": Field(
+            config=Bool, is_required=False, default_value=True, description="FIXME"
+        ),
     },
+    tags={"kind": "dbt"},
 )
 @experimental
-def dbt_cli_compile(context) -> DbtCliResult:
-    logs, raw_output, return_code = execute_dbt(
+def dbt_cli_compile(context) -> Dict:
+    """This solid executes ``dbt compile`` via the dbt CLI."""
+    cli_output = execute_cli(
         context.solid_config["dbt_executable"],
         command=("compile",),
         flags_dict=passthrough_flags_only(
@@ -491,21 +527,11 @@ def dbt_cli_compile(context) -> DbtCliResult:
         ignore_handled_error=context.solid_config["ignore_handled_error"],
     )
 
-    yield AssetMaterialization(
-        asset_key="dbt_cli_compile-shell_output",  # TODO: Perhaps derive asset key from CLI flags?
-        description="The output of a shell execution of `dbt compile`.",
-        metadata_entries=[
-            EventMetadataEntry.float(
-                label="return_code",
-                value=float(return_code),
-                description="The return code of a shell exeuction of `dbt compile`.",
-            ),
-            EventMetadataEntry.text(
-                label="raw_output",
-                text=raw_output,
-                description="The raw output of a shell execution of `dbt compile`.",
-            ),
-        ],
-    )
+    if context.solid_config["yield_materializations"]:
+        yield AssetMaterialization(
+            asset_key="dbt_compile_cli_output",
+            description="Output from the CLI execution of `dbt compile`.",
+            metadata_entries=[EventMetadataEntry.json(cli_output, label="CLI Output")],
+        )
 
-    yield Output(DbtCliResult(logs=logs, raw_output=raw_output, return_code=return_code))
+    yield Output(cli_output)
